@@ -31,6 +31,7 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -42,10 +43,13 @@
 #include <signal.h>
 #include <OpenIPMI/selector.h>
 #include <OpenIPMI/ipmi_conn.h>
+#include <OpenIPMI/ipmi_err.h>
 #include <OpenIPMI/ipmi_posix.h>
 #include <OpenIPMI/ipmi_glib.h>
+#include <OpenIPMI/ipmi_tcl.h>
 #include <OpenIPMI/ipmi_cmdlang.h>
 #include <OpenIPMI/ipmi_debug.h>
+#include <editline/readline.h>
 
 #ifdef HAVE_GLIB
 #include <glib.h>
@@ -77,9 +81,6 @@ os_hnd_fd_id_t *term_fd_id;
 static int done = 0;
 static int evcount = 0;
 static int handling_input = 0;
-static char *line_buffer;
-static int  line_buffer_max = 0;
-static int  line_buffer_pos = 0;
 static int cmd_redisp = 1;
 
 static void user_input_ready(int fd, void *data, os_hnd_fd_id_t *id);
@@ -92,8 +93,7 @@ redraw_cmdline(int force)
     if (force)
 	redisp = 1;
     if (!done && handling_input && redisp) {
-	fputs("> ", stdout);
-	fwrite(line_buffer, 1, line_buffer_pos, stdout);
+	rl_redisplay();
 	fflush(stdout);
     }
 }
@@ -436,7 +436,7 @@ out_binary(ipmi_cmdlang_t *info, const char *name, const char *value,
     out_data_t *out_data = info->user_data;
     unsigned char *data = (unsigned char *) value;
     int indent2 = (out_data->indent * 2) + strlen(name) + 1;
-    int i;
+    unsigned int i;
     char *sep = ":";
 
     if (info->help)
@@ -509,16 +509,19 @@ cmd_done(ipmi_cmdlang_t *info)
     out_data_t *out_data = info->user_data;
 
     if (info->err) {
+	char errval[128];
 	if (!info->location)
 	    info->location = "";
 	if (strlen(info->objstr) == 0) {
-	    fprintf(out_data->stream, "error: %s: %s (0x%x)\n",
+	    fprintf(out_data->stream, "error: %s: %s (0x%x, %s)\n",
 		    info->location, info->errstr,
-		    info->err);
+		    info->err,
+		    ipmi_get_error_string(info->err, errval, sizeof(errval)));
 	} else {
-	    fprintf(out_data->stream, "error: %s %s: %s (0x%x)\n",
+	    fprintf(out_data->stream, "error: %s %s: %s (0x%x, %s)\n",
 		    info->location, info->objstr, info->errstr,
-		    info->err);
+		    info->err,
+		    ipmi_get_error_string(info->err, errval, sizeof(errval)));
 	}
 	if (info->errstr_dynalloc)
 	    ipmi_mem_free(info->errstr);
@@ -564,7 +567,7 @@ ipmi_cmdlang_report_event(ipmi_cmdlang_event_t *event)
     enum ipmi_cmdlang_out_types type;
     char                        *name, *value;
     int                         indent2;
-    int                         i;
+    unsigned int                i;
 
     if (handling_input && !done && cmd_redisp)
 	fputc('\n', stdout);
@@ -603,105 +606,41 @@ ipmi_cmdlang_report_event(ipmi_cmdlang_event_t *event)
 static void
 user_input_ready(int fd, void *data, os_hnd_fd_id_t *id)
 {
-    ipmi_cmdlang_t *info = data;
-    out_data_t *out_data = info->user_data;
-    char rc;
-    int  count;
-    int  i;
+    rl_callback_read_char();
+}
 
-    count = read(fd, &rc, 1);
-    if (count <= 0) {
+static void
+rl_ipmish_cb_handler(char *cmdline)
+{
+    char *expansion = NULL;
+    int result;
+
+    if (cmdline == NULL) {
 	done = 1;
 	evcount = 1; /* Force a newline */
 	return;
     }
-
-    switch(rc) {
-    case 0x04: /* ^d */
-	if (line_buffer_pos == 0) {
-	    done = 1;
-	    evcount = 1; /* Force a newline */
-	}
-	break;
-
-    case 12: /* ^l */
-	fputc('\n', out_data->stream);
-	redraw_cmdline(1);
-	break;
-
-    case '\r': case '\n':
-	fputc(rc, out_data->stream);
-	if (line_buffer) {
-	    line_buffer[line_buffer_pos] = '\0';
-	    for (i=0; isspace(line_buffer[i]); i++)
-		;
-	    /* Ignore blank lines. */
-	    if (line_buffer[i] != '\0') {
-		/* Turn off input processing. */
-		disable_term_fd(info);
-
-		cmdlang.err = 0;
-		cmdlang.errstr = NULL;
-		cmdlang.errstr_dynalloc = 0;
-		cmdlang.location = NULL;
-		handling_input = 0;
-		line_buffer_pos = 0;
-		ipmi_cmdlang_handle(&cmdlang, line_buffer);
-	    } else {
-		fputs("> ", out_data->stream);
-	    }
-	} else {
-	    fputs("> ", out_data->stream);
-	}
-	break;
-
-    case 0x7f: /* delete */
-    case '\b': /* backspace */
-	if (line_buffer_pos > 0) {
-	    line_buffer_pos--;
-	    fputs("\b \b", out_data->stream);
-	}
-	break;
-
-    default:
-	if (line_buffer_pos >= line_buffer_max) {
-	    char *new_line = ipmi_mem_alloc(line_buffer_max+10+1);
-	    if (!new_line)
-		break;
-	    line_buffer_max += 10;
-	    if (line_buffer) {
-		memcpy(new_line, line_buffer, line_buffer_pos);
-		ipmi_mem_free(line_buffer);
-	    }
-	    line_buffer = new_line;
-	}
-	line_buffer[line_buffer_pos] = rc;
-	line_buffer_pos++;
-	fputc(rc, out_data->stream);
-	break;
+    result = history_expand(cmdline, &expansion);
+    if (result < 0 || result == 2) {
+	fprintf(stderr, "%s\n", expansion);
+    } else if (expansion && strlen(expansion)){
+	cmdlang.err = 0;
+	cmdlang.errstr = NULL;
+	cmdlang.errstr_dynalloc = 0;
+	cmdlang.location = NULL;
+	handling_input = 0;
+	add_history(expansion);
+	ipmi_cmdlang_handle(&cmdlang, expansion);
     }
-
-    fflush(out_data->stream);
+    if (expansion)
+	free(expansion);
 }
 
-static int term_setup;
-struct termios old_termios;
-
 static void
 cleanup_term(void)
 {
-    if (line_buffer) {
-	ipmi_mem_free(line_buffer);
-	line_buffer = NULL;
-    }
+    rl_callback_handler_remove();
     disable_term_fd(&cmdlang);
-
-    if (!term_setup)
-	return;
-
-    tcsetattr(0, TCSADRAIN, &old_termios);
-    tcdrain(0);
-    term_setup = 0;
 }
 
 static void cleanup_sig(int sig);
@@ -709,20 +648,14 @@ static void cleanup_sig(int sig);
 static void
 setup_term(os_handler_t *os_hnd)
 {
-    struct termios new_termios;
-
-    tcgetattr(0, &old_termios);
-    new_termios = old_termios;
-    new_termios.c_lflag &= ~(ICANON|ECHO);
-    tcsetattr(0, TCSADRAIN, &new_termios);
-    term_setup = 1;
-
     signal(SIGINT, cleanup_sig);
     signal(SIGPIPE, cleanup_sig);
     signal(SIGUSR1, cleanup_sig);
     signal(SIGUSR2, cleanup_sig);
     signal(SIGPWR, cleanup_sig);
 
+    stifle_history(500);
+    rl_callback_handler_install("> ", rl_ipmish_cb_handler);
     lout_data.stream = stdout;
 
     cmdlang.os_hnd = os_hnd;
@@ -805,6 +738,7 @@ read_cmd(ipmi_cmd_info_t *cmd_info)
     read_nest++;
     saved_done_ptr = done_ptr;
 
+    /* not record the file's commands into history */
     while (fgets(cmdline, sizeof(cmdline), s)) {
 	my_out_data.stream = stdout;
 	my_out_data.indent = 0;
@@ -812,6 +746,7 @@ read_cmd(ipmi_cmd_info_t *cmd_info)
 	cdone = 0;
 	done_ptr = &cdone;
 	printf("> %s", cmdline);
+	fflush(stdout);
 	ipmi_cmdlang_handle(&my_cmdlang, cmdline);
 	while (!cdone)
 	    cmdlang->os_hnd->perform_one_op(cmdlang->os_hnd, NULL);
@@ -844,7 +779,7 @@ setup_cmds(void)
 			      "on|off - If an asynchronous event comes in,"
 			      " redisplay the current working command.  This"
 			      " is on by default.",
-			      redisp_cmd, NULL, NULL);
+			      redisp_cmd, NULL, NULL, NULL);
     if (rv) {
 	fprintf(stderr, "Error adding exit command: 0x%x\n", rv);
 	exit(1);
@@ -853,7 +788,7 @@ setup_cmds(void)
     rv = ipmi_cmdlang_reg_cmd(NULL,
 			      "exit",
 			      "- leave the program",
-			      exit_cmd, NULL, NULL);
+			      exit_cmd, NULL, NULL, NULL);
     if (rv) {
 	fprintf(stderr, "Error adding exit command: 0x%x\n", rv);
 	exit(1);
@@ -863,7 +798,7 @@ setup_cmds(void)
 			      "read",
 			      "<file> - Read commands from the file and"
 			      " execute them",
-			      read_cmd, NULL, NULL);
+			      read_cmd, NULL, NULL, NULL);
     if (rv) {
 	fprintf(stderr, "Error adding read command: 0x%x\n", rv);
 	exit(1);
@@ -942,6 +877,9 @@ static char *usage_str =
 #ifdef HAVE_GLIB
 "  --glib - use glib for the OS handler.\n"
 #endif
+#ifdef HAVE_TCL
+"  --tcl - use tcl for the OS handler.\n"
+#endif
 #ifdef HAVE_UCDSNMP
 "  --snmp - turn on SNMP trap handling.\n"
 #endif
@@ -968,6 +906,9 @@ main(int argc, char *argv[])
 #ifdef HAVE_GLIB
     int              use_glib = 0;
 #endif
+#ifdef HAVE_TCL
+    int              use_tcl = 0;
+#endif
 
     colstr = getenv("COLUMNS");
     if (colstr) {
@@ -1008,6 +949,10 @@ main(int argc, char *argv[])
 #ifdef HAVE_GLIB
 	} else if (strcmp(arg, "--glib") == 0) {
 	    use_glib = 1;
+#endif
+#ifdef HAVE_TCL
+	} else if (strcmp(arg, "--tcl") == 0) {
+	    use_tcl = 1;
 #endif
 	} else if (strcmp(arg, "--help") == 0) {
 	    usage(argv[0]);
@@ -1036,7 +981,7 @@ main(int argc, char *argv[])
 	sel = NULL;
 #endif
 	g_thread_init(NULL);
-	os_hnd = ipmi_glib_get_os_handler();
+	os_hnd = ipmi_glib_get_os_handler(0);
 	if (!os_hnd) {
 	    fprintf(stderr,
 		    "ipmi_smi_setup_con: Unable to allocate os handler\n");
@@ -1052,6 +997,19 @@ main(int argc, char *argv[])
 			  | G_LOG_FLAG_FATAL,
 			  glib_handle_log,
                           NULL);
+#endif
+#ifdef HAVE_TCL
+    } else if (use_tcl) {
+#ifdef HAVE_UCDSNMP
+	init_snmp = 0; /* No SNMP support for tcl yet. */
+	sel = NULL;
+#endif
+	os_hnd = ipmi_tcl_get_os_handler(0);
+	if (!os_hnd) {
+	    fprintf(stderr,
+		    "ipmi_smi_setup_con: Unable to allocate os handler\n");
+	    return 1;
+	}
 #endif
     } else {
 	os_hnd = ipmi_posix_setup_os_handler();
@@ -1093,8 +1051,9 @@ main(int argc, char *argv[])
 	read_nest = 1;
 	execs = e->next;
 	printf("> %s\n", e->str);
+	fflush(stdout);
 	done_ptr = &cdone;
-	ipmi_cmdlang_handle(&cmdlang, e->str);
+	rl_ipmish_cb_handler(e->str);
 	while (!cdone)
 	    os_hnd->perform_one_op(os_hnd, NULL);
 	done_ptr = NULL;
@@ -1102,7 +1061,6 @@ main(int argc, char *argv[])
 	read_nest = 0;
     }
 
-    printf("> ");
     fflush(stdout);
 
     handling_input = 1;
@@ -1127,8 +1085,11 @@ main(int argc, char *argv[])
 
     os_hnd->free_os_handler(os_hnd);
 
+    /* remove the prompt which editline printed */
+    printf("\b\b  \b\b");
     if (evcount)
 	printf("\n");
+    fflush(stdout);
 
     if (rv)
 	return 1;
