@@ -31,6 +31,8 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <config.h>
+
 /* Get the rwlocks for GNU. */
 #define _GNU_SOURCE
 
@@ -50,6 +52,7 @@
 #endif
 
 #include <OpenIPMI/os_handler.h>
+#include <OpenIPMI/ipmi_glib.h>
 
 #include <glib.h>
 
@@ -118,7 +121,7 @@ add_fd(os_handler_t       *handler,
     fd_data = g_malloc(sizeof(*fd_data));
     if (!fd_data)
 	return ENOMEM;
-    memset(fd_data, 0, sizeof(fd_data));
+    memset(fd_data, 0, sizeof(*fd_data));
     fd_data->chan = g_io_channel_unix_new(fd);
     if (!fd_data->chan) {
 	g_free(fd_data);
@@ -208,6 +211,7 @@ stop_timer(os_handler_t *handler, os_hnd_timer_id_t *id)
     if (!id->running)
 	return EINVAL;
 
+    id->running = 0;
     g_source_remove(id->ev_id);
 
     return 0;
@@ -691,7 +695,7 @@ database_store(os_handler_t  *handler,
 
     gkey.dptr = key;
     gkey.dsize = strlen(key);
-    gdata.dptr = data;
+    gdata.dptr = (char *) data;
     gdata.dsize = data_len;
 
     rv = gdbm_store(info->gdbmf, gkey, gdata, GDBM_REPLACE);
@@ -731,7 +735,7 @@ database_find(os_handler_t  *handler,
     g_mutex_unlock(info->gdbm_lock);
     if (!gdata.dptr)
 	return EINVAL;
-    *data = gdata.dptr;
+    *data = (unsigned char *) gdata.dptr;
     *data_len = gdata.dsize;
     *fetch_completed = 1;
     return 0;
@@ -760,8 +764,8 @@ set_gdbm_filename(os_handler_t *os_hnd, char *name)
 }
 #endif
 
-void sset_log_handler(os_handler_t *handler,
-		      os_vlog_t    log_handler)
+static void sset_log_handler(os_handler_t *handler,
+			     os_vlog_t    log_handler)
 {
     g_os_hnd_data_t *info = handler->internal_data;
 
@@ -816,12 +820,16 @@ static os_handler_t ipmi_glib_os_handler =
     .set_log_handler = sset_log_handler,
 };
 
+
 os_handler_t *
 ipmi_glib_get_os_handler(int priority)
 {
     os_handler_t    *rv;
     g_os_hnd_data_t *info;
 
+    if (!g_thread_supported ())
+	g_thread_init(NULL);
+
     rv = g_malloc(sizeof(*rv));
     if (!rv)
 	return NULL;
@@ -849,3 +857,49 @@ ipmi_glib_get_os_handler(int priority)
 
     return rv;
 }
+
+static void (*log_hndlr)(const char *domain, const char *pfx, const char *msg);
+
+static void
+glib_handle_log(const gchar *log_domain,
+		GLogLevelFlags log_level,
+		const gchar *message,
+		gpointer user_data)
+{
+    void (*hndlr)(const char *domain, const char *pfx, const char *msg);
+    char *pfx = "";
+    if (log_level & G_LOG_LEVEL_ERROR)
+	pfx = "FATL";
+    else if (log_level & G_LOG_LEVEL_CRITICAL)
+	pfx = "SEVR";
+    else if (log_level & G_LOG_LEVEL_WARNING)
+	pfx = "WARN";
+    else if (log_level & G_LOG_LEVEL_MESSAGE)
+	pfx = "EINF";
+    else if (log_level & G_LOG_LEVEL_INFO)
+	pfx = "INFO";
+    else if (log_level & G_LOG_LEVEL_DEBUG)
+	pfx = "DEBG";
+
+    hndlr = log_hndlr;
+    if (hndlr)
+	hndlr(log_domain, pfx, message);
+}
+
+void
+ipmi_glib_set_log_handler(void (*hndlr)(const char *domain,
+					const char *pfx,
+					const char *msg))
+{
+    log_hndlr = hndlr;
+    g_log_set_handler("OpenIPMI",
+		      G_LOG_LEVEL_ERROR
+		      | G_LOG_LEVEL_CRITICAL
+		      | G_LOG_LEVEL_WARNING
+		      | G_LOG_LEVEL_MESSAGE
+		      | G_LOG_LEVEL_INFO
+		      | G_LOG_LEVEL_DEBUG
+		      | G_LOG_FLAG_FATAL,
+		      glib_handle_log,
+		      NULL);
+}
