@@ -519,12 +519,13 @@ ipmi_set_8_bit_ascii(const char    *input,
 }
 
 void
-ipmi_set_device_string(const char           *input,
-		       enum ipmi_str_type_e type,
-		       unsigned int         in_len,
-		       unsigned char        *output,
-		       int                  force_unicode,
-		       unsigned int         *out_len)
+ipmi_set_device_string2(const char           *input,
+			enum ipmi_str_type_e type,
+			unsigned int         in_len,
+			unsigned char        *output,
+			int                  force_unicode,
+			unsigned int         *out_len,
+			unsigned int         options)
 {
     const char   *s = input;
     int          bsize = 0; /* Start with 4-bit. */
@@ -538,15 +539,19 @@ ipmi_set_device_string(const char           *input,
 	in_len = 63;
 
     if (type == IPMI_ASCII_STR) {
-	for (i=0; i<in_len; i++) {
-	    if (table_4_bit[(int) *s] == 0) {
-		bsize |= 1;
-		if (table_6_bit[(int) *s] == 0) {
-		    bsize |= 2;
-		    break;
+	if (options && IPMI_STRING_OPTION_8BIT_ONLY)
+	    bsize = 2;
+	else {
+	    for (i=0; i<in_len; i++) {
+		if (table_4_bit[(int) *s] == 0) {
+		    bsize |= 1;
+		    if (table_6_bit[(int) *s] == 0) {
+			bsize |= 2;
+			break;
+		    }
 		}
+		s++;
 	    }
-	    s++;
 	}
 	if (bsize == 0) {
 	    /* We can encode it in 4-bit BCD+ */
@@ -570,6 +575,18 @@ ipmi_set_device_string(const char           *input,
     }
 }
 
+void
+ipmi_set_device_string(const char           *input,
+		       enum ipmi_str_type_e type,
+		       unsigned int         in_len,
+		       unsigned char        *output,
+		       int                  force_unicode,
+		       unsigned int         *out_len)
+{
+    ipmi_set_device_string2(input, type, in_len, output, force_unicode,
+			    out_len, IPMI_STRING_OPTION_NONE);
+}
+
 static long seq = 0;
 static os_hnd_lock_t *seq_lock;
 long
@@ -936,6 +953,8 @@ ipmi_init(os_handler_t *handler)
     if (rv)
 	return rv;
 
+    ipmi_initialized = 1;
+
     if (handler->create_lock) {
 	rv = handler->create_lock(handler, &seq_lock);
 	if (rv)
@@ -995,28 +1014,10 @@ ipmi_init(os_handler_t *handler)
     ipmi_oem_atca_init();
     init_oem_test();
 
-    ipmi_initialized = 1;
-
     return 0;
 
  out_err:
-    _ipmi_sol_shutdown();
-#ifdef HAVE_OPENIPMI_SMI
-    _ipmi_smi_shutdown();
-#endif
-    _ipmi_lan_shutdown();
-    ipmi_oem_intel_shutdown();
-    ipmi_oem_kontron_conn_shutdown();
-    _ipmi_mc_shutdown();
-    _ipmi_domain_shutdown();
-    _ipmi_fru_spd_decoder_shutdown();
-    _ipmi_normal_fru_shutdown();
-    _ipmi_fru_shutdown();
-    if (seq_lock)
-	handler->destroy_lock(ipmi_os_handler, seq_lock);
-    if (con_type_list)
-	locked_list_destroy(con_type_list);
-    ipmi_os_handler = NULL;
+    ipmi_shutdown();
     return rv;
 }
 
@@ -1037,6 +1038,7 @@ ipmi_shutdown(void)
     ipmi_oem_kontron_conn_shutdown();
     _ipmi_mc_shutdown();
     _ipmi_domain_shutdown();
+    _ipmi_fru_spd_decoder_shutdown();
     _ipmi_conn_shutdown();
     _ipmi_normal_fru_shutdown();
     _ipmi_fru_shutdown();
@@ -1045,6 +1047,8 @@ ipmi_shutdown(void)
     if (con_type_list)
 	locked_list_destroy(con_type_list);
 
+    ipmi_os_handler = NULL;
+
     ipmi_initialized = 0;
 }
 
