@@ -31,6 +31,8 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <config.h>
+
 /* Get the rwlocks for GNU. */
 #define _GNU_SOURCE
 
@@ -57,7 +59,7 @@
 
 /* CHEAP HACK - we don't want the user to have to provide this any
    more. */
-extern void posix_vlog(const char           *format,
+extern void posix_vlog(char                 *format,
 		       enum ipmi_log_type_e log_type,
 		       va_list              ap);
 #pragma weak posix_vlog
@@ -267,6 +269,7 @@ get_random(os_handler_t *handler, void *data, unsigned int len)
 	    goto out;
 	}
 	len -= rv;
+	data += rv;
     }
 
     rv = 0;
@@ -337,7 +340,7 @@ sposix_vlog(os_handler_t         *handler,
     if (log_handler)
 	log_handler(handler, format, log_type, ap);
     else if (posix_vlog)
-	posix_vlog(format, log_type, ap);
+	posix_vlog((char *) format, log_type, ap);
     else
 	default_vlog(format, log_type, ap);
 }
@@ -351,7 +354,7 @@ sposix_log(os_handler_t         *handler,
     va_list ap;
 
     va_start(ap, format);
-    posix_vlog(format, log_type, ap);
+    sposix_vlog(handler, log_type, format, ap);
     va_end(ap);
 }
 
@@ -392,7 +395,8 @@ destroy_lock(os_handler_t  *handler,
     int rv;
 
     if (id->lock_count != 0)
-	ipmi_log(IPMI_LOG_FATAL, "Destroy of lock when count is not zero");
+	handler->log(handler, IPMI_LOG_FATAL,
+		     "Destroy of lock when count is not zero");
     rv = pthread_mutex_destroy(&id->mutex);
     if (rv)
 	return rv;
@@ -423,9 +427,9 @@ unlock(os_handler_t  *handler,
     int rv;
 
     if (id->lock_count == 0)
-	ipmi_log(IPMI_LOG_FATAL, "lock count went negative");
+	handler->log(handler, IPMI_LOG_FATAL, "lock count went negative");
     if (pthread_self() != id->owner)
-	ipmi_log(IPMI_LOG_FATAL, "lock release by non-owner");
+	handler->log(handler, IPMI_LOG_FATAL, "lock release by non-owner");
     id->lock_count--;
     if (id->lock_count == 0) {
 	rv = pthread_mutex_unlock(&id->mutex);
@@ -482,7 +486,17 @@ cond_wait(os_handler_t  *handler,
 	  os_hnd_cond_t *cond,
 	  os_hnd_lock_t *lock)
 {
-    return pthread_cond_wait(&cond->cond, &lock->mutex);
+    int       rv;
+    int       old_lock_count;
+    pthread_t old_owner;
+
+    old_lock_count = lock->lock_count; 
+    old_owner = lock->owner;
+    lock->lock_count = 0;
+    rv = pthread_cond_wait(&cond->cond, &lock->mutex);
+    lock->lock_count = old_lock_count;
+    lock->owner = old_owner;
+    return rv;
 }
 
 static int
@@ -493,6 +507,9 @@ cond_timedwait(os_handler_t   *handler,
 {
     struct timespec spec;
     struct timeval  now;
+    int             rv;
+    int             old_lock_count;
+    pthread_t       old_owner;
 
     gettimeofday(&now, NULL);
     spec.tv_sec = timeout->tv_sec + now.tv_sec;
@@ -501,7 +518,13 @@ cond_timedwait(os_handler_t   *handler,
 	spec.tv_sec += 1;
 	spec.tv_nsec -= 1000000000;
     }
-    return pthread_cond_timedwait(&cond->cond, &lock->mutex, &spec);
+    old_lock_count = lock->lock_count; 
+    old_owner = lock->owner;
+    lock->lock_count = 0;
+    rv = pthread_cond_timedwait(&cond->cond, &lock->mutex, &spec);
+    lock->lock_count = old_lock_count;
+    lock->owner = old_owner;
+    return rv;
 }
 
 static int
@@ -695,7 +718,7 @@ database_store(os_handler_t  *handler,
 
     gkey.dptr = key;
     gkey.dsize = strlen(key);
-    gdata.dptr = data;
+    gdata.dptr = (char *) data;
     gdata.dsize = data_len;
 
     rv = gdbm_store(info->gdbmf, gkey, gdata, GDBM_REPLACE);
@@ -735,7 +758,7 @@ database_find(os_handler_t  *handler,
     pthread_mutex_unlock(&info->gdbm_lock);
     if (!gdata.dptr)
 	return EINVAL;
-    *data = gdata.dptr;
+    *data = (unsigned char *) gdata.dptr;
     *data_len = gdata.dsize;
     *fetch_completed = 1;
     return 0;
@@ -764,8 +787,8 @@ set_gdbm_filename(os_handler_t *os_hnd, char *name)
 }
 #endif
 
-void sset_log_handler(os_handler_t *handler,
-		      os_vlog_t    log_handler)
+static void sset_log_handler(os_handler_t *handler,
+			     os_vlog_t    log_handler)
 {
     pt_os_hnd_data_t *info = handler->internal_data;
 
@@ -814,7 +837,7 @@ ipmi_posix_thread_get_os_handler(void)
 {
     os_handler_t     *rv;
     pt_os_hnd_data_t *info;
-#if HAVE_GDBM
+#ifdef HAVE_GDBM
     int              err;
 #endif
 
@@ -880,43 +903,6 @@ ipmi_posix_thread_setup_os_handler(int wake_sig)
     return os_hnd;
 }
 
-/*
- * Support for the selector code.
- */
-
-int
-posix_mutex_alloc(void **val)
-{
-    void *m = malloc(sizeof(pthread_mutex_t));
-    int  rv;
-
-    if (!m)
-	return ENOMEM;
-
-    rv = pthread_mutex_init(m, NULL);         
-    if (rv) {
-	free(m);
-	return rv;
-    }
-    *val = m;
-    return 0;
-}
-
-void posix_mutex_free(void *val)
-{
-    free(val);
-}
-
-void posix_mutex_lock(void *val)
-{
-    pthread_mutex_lock(val);
-}
-
-void posix_mutex_unlock(void *val)
-{
-    pthread_mutex_unlock(val);
-}
-
 /*
  * Cruft below, do not use these any more.
  */
